#!/bin/sh

export hunspell_VERSION=1.7.0
export hunspell_ARCHIVE=v${hunspell_VERSION}.tar.gz
export hunspell_DIRNAME=hunspell-${hunspell_VERSION}
export hunspell_URL=https://github.com/hunspell/hunspell/archive/${hunspell_ARCHIVE}

export poppler_VERSION=0.85.0
export poppler_ARCHIVE=poppler-${poppler_VERSION}.tar.xz
export poppler_DIRNAME=poppler-${poppler_VERSION}
export poppler_URL=https://poppler.freedesktop.org/${poppler_ARCHIVE}

export popplerdata_VERSION=0.4.9
export popplerdata_ARCHIVE=poppler-data-${popplerdata_VERSION}.tar.gz
export popplerdata_DIRNAME=poppler-data-${popplerdata_VERSION}
export popplerdata_URL=https://poppler.freedesktop.org/${popplerdata_ARCHIVE}

