[
  "../../anthosthesis.cls",
  "./a file containing spaces.aux",
  "./a file containing spaces.tex",
  "./a file with spaces.aux",
  "./a file with spaces.tex",
  "./another-very-very-long-filename.aux",
  "./anthosthesis-example.tex",
  "./captions=tableheading.tex",
  "./captions=tableheading.aux",
  "./many-files-on-a-single-line2.tex",
  "./many-files-on-a-single-line4444444.tex",
  "./many-files-on-a-single-line5.tex",
  "./many-files-on-a-single-line66666666.tex",
  "./parenthesis (così) inside a filename just to complicate your life (and my life too).tex",
  "./più file su una singola linea2.tex",
  "./più file su una singola linea4.tex",
  "./più file su una singola linea5.tex",
  "./più file su una singola linea66.tex",
  "./plot-f(x).tex",
  "./prova.aux",
  "./prova.tex",
  "./relazione.aux",
  "./relazione.tex",
  "./relazione.toc",
  "./rightname.sty",
  "./show-long-macro.tex",
  "./show-long-macro.aux",
  "./10.tex",
  "./untitled-1.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/babel/babel.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/babel/italian.ldf",
  "/usr/local/texlive/2011/texmf-dist/tex/context/base/supp-pdf.mkii",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/etexcmds.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/hobsub-hyperref.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/ifluatex.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/ifpdf.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/infwarerr.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/kvdefinekeys.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/kvsetkeys.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/ltxcmds.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/oberdiek/pdftexcmds.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcore.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorearrows.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcoreexternal.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcoregraphicstate.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcoreimage.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorelayers.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcoreobjects.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorepathconstruct.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorepathprocessing.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorepathusage.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorepatterns.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorepoints.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorequick.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcorescopes.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcoreshade.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcoretransformations.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/basiclayer/pgfcoretransparency.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/frontendlayer/tikz/libraries/tikzlibrarytopaths.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/frontendlayer/tikz/tikz.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/libraries/pgflibraryplothandlers.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmath.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathcalc.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfloat.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.base.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.basic.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.comparison.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.misc.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.random.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.round.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathfunctions.trigonometric.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathparser.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/math/pgfmathutil.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/modules/pgfmodulematrix.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/modules/pgfmoduleplot.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/modules/pgfmoduleshapes.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/systemlayer/pgf.cfg",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/systemlayer/pgfsys-common-pdf.def",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/systemlayer/pgfsys-pdftex.def",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/systemlayer/pgfsys.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/systemlayer/pgfsysprotocol.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/systemlayer/pgfsyssoftpath.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/utilities/pgffor.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/utilities/pgfkeys.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/utilities/pgfkeysfiltered.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/utilities/pgfrcs.code.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/utilities/pgfutil-common.tex",
  "/usr/local/texlive/2011/texmf-dist/tex/generic/pgf/utilities/pgfutil-latex.def",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/article.cls",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/book.cls",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/fontenc.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/inputenc.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/omsenc.dfu",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/ot1enc.dfu",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/size10.clo",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/t1cmss.fd",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/t1enc.def",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/t1enc.dfu",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/base/utf8.def",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/graphics/dvipsnam.def",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/graphics/graphics.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/graphics/graphicx.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/graphics/keyval.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/graphics/trig.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/hyperref/hpdftex.def",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/hyperref/hyperref.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/hyperref/nameref.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/scrartcl.cls",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/scrbase.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/scrbook.cls",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/scrkbase.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/scrlfile.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/scrreprt.cls",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/scrsize10pt.clo",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/tocbasic.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/koma-script/typearea.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/latexconfig/color.cfg",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/latexconfig/epstopdf-sys.cfg",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/latexconfig/graphics.cfg",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/microtype/microtype.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/ms/everyshi.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/oberdiek/bkm-pdftex.def",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/oberdiek/bookmark.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/oberdiek/epstopdf-base.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/oberdiek/grfext.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/oberdiek/kvoptions.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pdftex-def/pdftex.def",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/basiclayer/pgf.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/basiclayer/pgfcore.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/compatibility/pgfcomp-version-0-65.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/compatibility/pgfcomp-version-1-18.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/frontendlayer/tikz.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/systemlayer/pgfsys.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/utilities/pgffor.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/utilities/pgfkeys.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/pgf/utilities/pgfrcs.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/xcolor/xcolor.sty",
  "/usr/local/texlive/2011/texmf-dist/tex/latex/extsizes/extarticle.cls"
]
