// TeXworksScript
// Title: Tw.insertText test
// Description: This is a unicode string 🤩
// Author: Stefan Löffler
// Version: 0.0.1
// Date: 2019-10-22
// Script-Type: standalone
// Context: TeXDocument
// Shortcut: Ctrl+Alt+Shift+I

TW.target.insertText("It works!");

[1, 2, 3];
